local MLTPlugin = getMLTPluginManager()
local scene = getCurrentScene()
local componentInterface = getComponentInterface()
local camera = Object3D(scene:getObjectByName("camera"))
local viewport = Viewport(scene:getObjectByName("viewport"))

g_trackerPath = "../tracker/tracker.xml"
g_trackingIndex = -1
g_targetCount = -1
g_targetIndex = -1
g_keyFrame = -1
g_debug = false

local isSVInit = false
local err_ret = eOk
local position = Vector3()
local orientation = Quaternion()
local kfIndex = -1
local tFlag = false
local jsNotify = 0
local onPlay = true


local sound = Sound(scene:getObjectByName("readygo"));

local t_array = {}
local max_index = 99
local temp_index = 1

for i = 0,max_index do
	texture_name = "texture_" .. temp_index
	t_array[i] = Texture(scene:getObjectByName(texture_name))
	temp_index = temp_index + 1
end

local temp_num = 1
local index = 0

local function setTextureToPlane(plane, texture)
	local planeMaterial = plane:getMaterial(0)
	planeMaterial:setSceneBlending(TI_SBT_SOURCE_ALPHA, TI_SBT_ONE_MINUS_SOURCE_ALPHA)
	planeMaterial:setHardwareCullingMode(TI_HW_CULL_CLOCKWISE)
	planeMaterial:setLightingEnabled(false)
	planeMaterial:setTexture(texture)
end

-- Don't remove this callback!
eventCallback(0, "SYSTEM", SCENARIO_STARTED, "", "")

coroutine.yield()

function updateTracker(url, sruTrackerPath, targetDir)
	local targetDirectory = ""
	if targetDir == null then
		--targetDirectory = getUserAppDataDirectory() .. "sv_default/remote/"
		targetDirectory = getUserAppDataDirectory()
	else
		targetDirectory = targetDir
	end
	
	eventCallback(0, "EVENT", TRACKER_UPDATE_STARTED, "", "")

	local msg = ""
	msg = "Starting download..."
	LOG(msg);
	
	local targetDistant = url
	local id = startRemoteUpdate( targetDistant, targetDirectory )
	if ( id > 0 ) then
		msg = "Download started, in progress..."
		LOG(msg)
		local status = 0
		local failed = false
		local finish = false
		repeat
			status = getRemoteUpdateStatus( id )
			if status==TI_DLFINISHED then 
				msg = "Download finsihed successfully"
				LOG(msg)
				eventCallback(0, "EVENT", TRACKER_UPDATE_DLFINISHED, "", "")
				finish = true
				failed = false
			elseif status==TI_DLFAILED then
				eventCallback(0, "EVENT", TRACKER_UPDATE_FAILED, "", "")
				msg = "Error: Download failed."
				LOG(msg)
				finish = true
				failed = true
			elseif status==TI_DLCANCELED then
				eventCallback(0, "EVENT", TRACKER_UPDATE_CANCELED, "", "")
				msg = "Error: Download canceled."
				LOG(msg)
				finish = true
				failed = true
			end
			LOG("status is "..status)
			coroutine.yield()
		until finish
	else 
		msg = "Error : Failed to start download."
		eventCallback(0, "EVENT", TRACKER_UPDATE_START_FAILED, "", "")
		LOG(msg)
		failed = true
	end
	if failed == false then
		if sruTrackerPath ~= null then
		    targetDirectory = targetDirectory .. sruTrackerPath
		else
			targetDirectory = targetDirectory .."tracker.zip/tracker.xml"
		end
		LOG(targetDirectory)
		return targetDirectory 
	else
		return "../tracker/tracker.xml"
	end
end

function changeMode(command)
	LOG("changeMode")
	local reqCode = command["arg0"]
	local cArg1 = command["arg1"]
	if cArg1 == "SV" then
		if g_mode ~= MODE_SV then 
			if g_trackingIndex > -1 then
				MLTPlugin:stopTracking(g_trackingIndex)
			end
			eventCallback(0, "NOTIFY", INITMODE_SV, "", "")
			g_mode = MODE_SV
			g_trackerPath = "../tracker/tracker.xml"
			err_ret, g_trackingIndex = MLTPlugin:startTracking(g_trackerPath, 0, camera)
		end
		err_ret,status = MLTPlugin:getTrackingStatus(g_trackingIndex)
		err_ret, g_targetCount = MLTPlugin:getTargetCount(g_trackingIndex)
		isSVInit = true
	elseif cArg1 == "SV_UPD" then
		if g_trackingIndex > -1 then
			MLTPlugin:stopTracking(g_trackingIndex)
		end
		eventCallback(0, "NOTIFY", INITMODE_SV_UPD, "", "")
		g_mode = MODE_SV_UPD
		url = command["arg2"]
		sruTrackerPath = command["arg3"]
		targetDir = command["arg4"]
		if cArg1_url ~= null then
			g_trackerPath = updateTracker(url, sruTrackerPath, targetDir)
			LOG(g_trackerPath)
		else
			g_trackerPath = "../tracker/tracker.xml"
		end
		err_ret, g_trackingIndex = MLTPlugin:startTracking(g_trackerPath, 0, camera)
		err_ret,status = MLTPlugin:getTrackingStatus(g_trackingIndex)
		err_ret, g_targetCount = MLTPlugin:getTargetCount(g_trackingIndex)
		isSVInit = true
	elseif cArg1 == null then
		eventCallback(0, "NOTIFY", INITMODE_NULL, "", "")
		g_mode = MODE_SV
		g_trackerPath = "../tracker/tracker.xml"
	else
		eventCallback(0, "NOTIFY", INITMODE_UNKNOWN, "", "")
		g_mode = MODE_SV
		g_trackerPath = "../tracker/tracker.xml"
	end
end


function handleCommand(command)
	local cName = command["CommandName"]
	local reqCode = command["arg0"] + 0
	if g_debug then
		local d_str = ""
		for key, value in pairs(command) do
			d_str = d_str.."key:"..key..",value:"..value.."\n"
		end
		eventCallback(0, "NOTIFY", DEBUG_NOTIFY, d_str, "")
	end
	if cName == "MODE_SET" then
		changeMode(command)
	elseif cName == "TOUCH_EVENT" then
		-- For iPhone
		local t	= command["arg1"]+0
		local x	= command["arg2"]+0
		local y	= command["arg3"]+0
		local obj = viewport:pick(x, y)
		if obj:isNull() then
			LOG("XXXXXXX Touch obj in SATCH is null")
		else
			LOG("XXXXXXX Touch obj in SATCH is "..obj:getName())
			if t > 0 then
				touchCallback(g_targetIndex, g_keyFrame, obj:getName(), t, x, y)
			end
		end
	elseif cName == "TAKE_PICTURE" then
		LOG("Take Picture")
		targetDirectory = command["arg1"]
		targetFile = command["arg2"]
		targetJpg = targetDirectory .. targetFile
		local mainView = RenderTarget(scene:getMainView())
		mainView:dump(targetJpg)
		luaTakePictureCallback(targetJpg)
	elseif cName == "KEYFRAME" then
		if command["arg1"] ~= null then
			local tIdx = command["arg1"]+1
			local flag = command["arg2"]
			if tIdx == 0 and not (g_keyFrame < 0) then
				tIdx = g_keyFrame+1
			end
			if tIdx >0 and tIdx <= #targets then
				if flag == "ON" then
					targets[tIdx]['status'] = true
					LOG("targets["..tIdx.."] is activated")
					eventCallback(reqCode, "NOTIFY", KEYFRAME_ON, "KEYFRAME_ON at "..tIdx, "")
				else
					targets[tIdx]['status'] = false
					LOG("targets["..tIdx.."] is deactivated")
					if targets[tIdx]['onPlay'] then
						if(targets[tIdx]['ref'] ~= null and targets[tIdx]['isScenette'])then
							if targets[tIdx]['ref']:getAnimationCount() > 0 then
								anim = targets[tIdx]['ref']:getAnimation(0)
								anim:stop()
							end
						end
						if targets[tIdx]['sound']~=null then
							targets[tIdx]['sound']:stop()
						end
						if targets[tIdx]['object'] ~= null then
							targets[tIdx]['object']:setVisible(false)
						end
						position:set(0, 0, 0)
						orientation:set(0, 0, 0, 0)
						targets[tIdx]['onPlay'] = false
					end
					eventCallback(reqCode, "NOTIFY", KEYFRAME_OFF, "", "")
				end
			else
				LOG("Out of range command : KEYFRAME")
				eventCallback(reqCode, "ERROR", OUT_OF_ARRAY, "OUT_OF_ARRAY at Lua KEYFRAME command", "")
			end
		else
			-- nothing to do
		end
	elseif cName == "JS_NOTIFY" then
		local flag = command["arg1"]
		if flag == "ON"  then
			jsNotify = 1
			eventCallback(reqCode, "NOTIFY", JS_NOTIFY_ON, "", "")
		else
			jsNotify = 0
			eventCallback(reqCode, "NOTIFY", JS_NOTIFY_OFF, "", "")
		end
	elseif cName == "SCALE" then
		local tIdx = command["arg1"]+1
		local scale = command["arg2"]+0.0
		if tIdx == 0 and not (g_keyFrame < 0) then
			tIdx = g_keyFrame+1
		end
		if tIdx > 0 and tIdx <= #targets then
			if scale < 0  then
				eventCallback(reqCode, "ERROR", ILLIGAL_PARAM, "ILLIGAL_PARAM at Lua SCALE command", "")
			else
				targets[tIdx]["scale"] = scale
				eventCallback(reqCode, "NOTIFY", SET_SCALE, "", "")
			end
		else
			eventCallback(reqCode, "ERROR", OUT_OF_ARRAY, "OUT_OF_ARRAY at Lua SCALE command", "")
		end
	elseif cName == "POSITION" then
		local tIdx = command["arg1"]+1
		local px = tonumber(command["arg2"])
		local py = tonumber(command["arg3"])
		local pz = tonumber(command["arg4"])
		if tIdx == 0 and not (g_keyFrame < 0) then
			tIdx = g_keyFrame+1
		end
		if tIdx > 0 and tIdx <= #targets then
			targets[tIdx]["pOffset"]=Vector3(px, py, pz)
			eventCallback(reqCode, "NOTIFY", SET_POSITION, "", "")
		else
			eventCallback(reqCode, "ERROR", OUT_OF_ARRAY, "OUT_OF_ARRAY at Lua POSITION command", "")
		end
	elseif cName == "ORIENTATION" then
		local tIdx = command["arg1"]+1
		local qw = tonumber(command["arg2"])
		local qx = tonumber(command["arg3"])
		local qy = tonumber(command["arg4"])
		local qz = tonumber(command["arg5"])
		if tIdx == 0 and not (g_keyFrame < 0) then
			tIdx = g_keyFrame+1
		end
		if tIdx > 0 and tIdx <= #targets then
			targets[tIdx]["qOffset"]=Quaternion(qx, qy, qz, qw)
			eventCallback(reqCode, "NOTIFY", SET_ORIENTATION, "", "")
		else
			eventCallback(reqCode, "ERROR", OUT_OF_ARRAY, "OUT_OF_ARRAY at Lua ORIENTATION command", "")
		end
	elseif cName == "DEBUG" then
		if g_debug then
			g_debug = false
			eventCallback(reqCode, "EVENT", DEBUG_END, "DEBUG_END", "")	
		else
			g_debug = true
			eventCallback(reqCode, "EVENT", DEBUG_START, "DEBUG_START", "")	
		end
	end
end

coroutine.yield()

--main routine

repeat
	isCommand, command = componentInterface:pullCommand()
	if isCommand then
		handleCommand(command)
	end

	if isSVInit == false then
		--check requireMode of startTracking. 4th param.
		err_ret, g_trackingIndex = MLTPlugin:startTracking(g_trackerPath, 0, camera)
		err_ret,status = MLTPlugin:getTrackingStatus(g_trackingIndex)
		err_ret, g_targetCount = MLTPlugin:getTargetCount(g_trackingIndex)
		isSVInit = true
	end

	for iTargetIndex =1, g_targetCount do
		err_ret, status = MLTPlugin:getTargetStatus(g_trackingIndex, iTargetIndex-1)
		if status > 0 then
			err_ret = MLTPlugin:getTargetPos(g_trackingIndex, iTargetIndex-1, position, orientation)
			err_ret, kfIndex = MLTPlugin:getRecognizedKeyFrameIndex(g_trackingIndex, iTargetIndex-1)
			
			local luaIndex = kfIndex+1
			local itemId = (targets[luaIndex]['itemId'] ~= null) and targets[luaIndex]['itemId'] or 0
			g_targetIndex = iTargetIndex-1
			trackingCallback(jsNotify, itemId, iTargetIndex-1, kfIndex, position, orientation)
			tFlag = true
			g_keyFrame = kfIndex
			
			-- If another target is detected, the target is stopped playing.
			for i =1, #targets do
				if i~= luaIndex then
					if targets[i]['onPlay'] then
						if(targets[i]['ref'] ~= null and targets[i]['isScenette']) then
							if targets[i]['ref']:getAnimationCount() > 0 then
								anim = targets[i]['ref']:getAnimation(0)
								anim:stop()
							end
						end
						if targets[i]['sound']~=null then
							targets[i]['sound']:stop()
						end
						if targets[i]['object']~=null then
							targets[i]['object']:setVisible(false)
						end
						targets[i]['onPlay'] = false
					end
				end
			end
			if targets[luaIndex]['status'] ~= false then
				if targets[luaIndex]['sound']~=null and targets[luaIndex]['onPlay']~=true then
					LOG(luaIndex.." is play sound")
					targets[luaIndex]['sound']:play()
				end
				
				-- The recognized target is started playing.
				if targets[luaIndex]['object']~=null then 
					if not targets[luaIndex]['object']:getVisible() then
						targets[luaIndex]['object']:setVisible(true)
						if targets[luaIndex]['has2d'] then
							--setTextureToPlane(targets[luaIndex]['plane2d'], targets[luaIndex]['texture'])
						
							if temp_num == 1 then
								
								setTextureToPlane(targets[luaIndex]['plane2d'], t_array[0])
							end
						end
						if(targets[luaIndex]['ref'] ~= null and targets[luaIndex]['isScenette']) then
							if targets[luaIndex]['ref']:getAnimationCount() > 0 then
								anim = targets[luaIndex]['ref']:getAnimation(0)
								anim:setLoop(true)
								anim:play()
							end
						end
					end
					targets[luaIndex]['object']:setScale(targets[luaIndex]["scale"])
					targets[luaIndex]['object']:setPosition(position+targets[luaIndex]["pOffset"], camera)
					targets[luaIndex]['object']:setOrientation(orientation*targets[luaIndex]["qOffset"], camera)
				end
				
				targets[luaIndex]['onPlay'] = true
			else
				--LOG("targets["..luaIndex.."] is not active")
			end
			
			temp_num = temp_num + 1
			
			--sound@start
			if temp_num == 50 then
				sound:play()
			end
			
			--animation start
			if temp_num >= 102 and index <= max_index then
				jouyo = temp_num % 2
				--if jouyo == 0 then
					setTextureToPlane(targets[luaIndex]['plane2d'], t_array[index])
					--LOG("index" .. index)
					index = index + 1
				--end
			end
			
		else
		-- If any targets aren't detected
		
			temp_num = 1
			index = 0
			
			for i =1, #targets do
				if targets[i]['onPlay'] then
					if(targets[i]['ref'] ~= null and targets[i]['isScenette'])then
						if targets[i]['ref']:getAnimationCount() > 0 then
							anim = targets[i]['ref']:getAnimation(0)
							anim:stop()
						end
					end
					if targets[i]['sound']~=null then
						targets[i]['sound']:stop()
					end
					if targets[i]['object'] ~= null then
						targets[i]['object']:setVisible(false)
					end
					position:set(0, 0, 0)
					orientation:set(0, 0, 0, 0)
					targets[i]['onPlay'] = false
				end
			end
			if tFlag == true then
				local luaIndex = g_keyFrame +1;
				if luaIndex > 0 and luaIndex <= #targets then
					position:set(0, 0, 0)
					orientation:set(0, 0, 0, 0)
					local itemId = (targets[luaIndex]['itemId'] ~= null) and targets[luaIndex]['itemId'] or 0
					trackingCallback(jsNotify, itemId, iTargetIndex-1, -1, position, orientation)
				else
					eventCallback(0, "ERROR", OUT_OF_ARRAY, "OUT_OF_ARRAY at Lua unFocusFlow : index "..luaIndex, "")
				end
				tFlag = false;
			end
			g_keyFrame = -1
		end
	end
until coroutine.yield()
